#!/usr/bin/env pybricks-micropython
from pybricks import ev3brick as brick
from pybricks.ev3devices import (Motor, TouchSensor, ColorSensor,
                                 InfraredSensor, UltrasonicSensor, GyroSensor)
from pybricks.parameters import (Port, Stop, Direction, Button, Color,
                                 SoundFile, ImageFile, Align)
from pybricks.tools import print, wait, StopWatch
from pybricks.robotics import DriveBase
from pybricks.iodevices import Ev3devSensor
from pybricks.hubs import EV3Brick
from pybricks.media.ev3dev import Font
import Keypoints
import random
keypoints = Keypoints.keypoints

#Инициализация оборудования  ********************
right_motor = Motor(Port.C, Direction.COUNTERCLOCKWISE)
left_motor = Motor(Port.B, Direction.COUNTERCLOCKWISE)

datchiki = Motor(Port.D, Direction.COUNTERCLOCKWISE)
catcher = Motor(Port.A, Direction.CLOCKWISE)

robot = DriveBase(left_motor, right_motor, 54, 135)

colorfild_S1 = ColorSensor(Port.S1)
colorfild_S4 = ColorSensor(Port.S4)
HiTechnic_S3 = Ev3devSensor(Port.S3)
# HiTechnic_S2 = Ev3devSensor(Port.S2)

#Инициализация переменных ********************
global position
whiteS1 = 33
whiteS4 = 32
blackS1 = 4
blackS4 = 4

diapS1 = whiteS1 - blackS1
diapS4 = whiteS4 - blackS4
error_old = 0
integral = 0

prop_coef = 0.125
integ_coef = 0.002
dif_coef = 0.04

power = 60
brake_coef = 0
robot.settings(500, 300, 300, 200)

robot_setup = dict()
robot_setup['catcher'] = "green_car"
robot_setup['bunker1'] = "blue_car"
robot_setup['bunker2'] = "red_car"


cars_position = dict()
cars_position['vidacha_1'] = 9
cars_position['vidacha_2'] = 9
cars_position['zaezd_1'] =   9       # "blue_car"
cars_position['zaezd_2'] =   9       # "green_car"
cars_position['zaezd_3'] =   9       # "red_car"
cars_position['zaezd_4'] =   9       # "green_car"
cars_position['zaezd_5'] =   9       # "blue_car"
cars_position['zaezd_6'] =   9       # "red_car"

cars_position['red_pos_1'] = 9
cars_position['red_pos_2'] = 9
cars_position['red_pos_3'] = 9
cars_position['red_pos_4'] = 9

cars_position['blue_pos_1'] = 9
cars_position['blue_pos_2'] = 9
cars_position['blue_pos_3'] = 9
cars_position['blue_pos_4'] = 9

cars_position['green_pos_1'] = 9
cars_position['green_pos_2'] = 9
cars_position['green_pos_3'] = 9
cars_position['green_pos_4'] = 9

cars_position['wall_pos_1'] = 9
cars_position['wall_pos_2'] = 9
cars_position['wall_pos_3'] = 9
cars_position['wall_pos_4'] = 9
cars_position['wall_pos_5'] = 9
cars_position['wall_pos_6'] = 9

cars_position['battery_1'] = 9
cars_position['battery_2'] = 9
cars_position['battery_3'] = 9
cars_position['battery_4'] = 9
#Функции ***************************************
def voltage_check():
    brick.screen.clear()
    big_font = Font(size=24, bold=True)
    brick.screen.set_font(big_font)
    voltage = brick.battery.voltage()
    print(voltage)
    brick.screen.print(voltage)
    if voltage <= 7600:
        brick.sound.beeps(10)

def brake():
    robot.stop(Stop.BRAKE)

def get_reflection_S1():
    color = ((colorfild_S1.reflection() - blackS1) / diapS1) * 100
    return color
 
def get_reflection_S4():

    color = ((colorfild_S4.reflection() - blackS4) / diapS4) * 100
    return color  

def calibration():
    while not Button.LEFT in brick.buttons():
        wait(10)
        whiteS1 = colorfild_S1.reflection() 
        whiteS4 = colorfild_S4.reflection()
        print('whites', whiteS1, whiteS4)

    while not Button.RIGHT in brick.buttons():
        wait(10)
        blackS1 = colorfild_S1.reflection() 
        blackS4 = colorfild_S4.reflection()
        print('blacks', blackS1, blackS4)

def drive_to(destination):
    global position
    while position != destination:

        instructions = keypoints[position][destination]
        print(instructions)
        next_pos = instructions[-1]
        instructions = instructions[0]
        for F in instructions:
            print(F)
            exec(F) 
 
        position = next_pos
        print("new pos",position)

def catcher_open():
    catcher.run_target(500, -350, then=Stop.HOLD, wait=True)

def catcher_catch():
    catcher.run_target(350, -180, then=Stop.HOLD, wait=True)

def catcher_high():
    catcher.run_target(900, -80, then=Stop.HOLD, wait=True)

def catcher_polet():
    catcher.run_target(500, -550, then=Stop.HOLD, wait=True)

def datchiki_open():
    datchiki.run_until_stalled(-600, then=Stop.BRAKE, duty_limit= -90)
    datchiki.dc(-10)
    wait(10)
    datchiki.stop(Stop.HOLD)

def datchiki_close():
    datchiki.run_until_stalled(600, then=Stop.BRAKE, duty_limit=90)
    datchiki.dc(10)
    wait(10)
    datchiki.stop(Stop.HOLD)

def pid_drive(integral, error_old, prop_coef, integ_coef, dif_coef, power):
    error = get_reflection_S1() - get_reflection_S4()
    integral = integral + error
    dif_error = error - error_old
    prop_reg_signal = error * prop_coef
    integ_reg_signal = integral * integ_coef
    dif_reg_signal = dif_error * dif_coef
    reg_signal = prop_reg_signal + integ_reg_signal + dif_reg_signal
    left_motor.dc(int(power + reg_signal))
    right_motor.dc(int(power - reg_signal))
    error_old = error
    wait(5)

def pid_T(angle, brake_coef, power):
    angle_now = left_motor.angle()
    while True:
        pid_drive(integral, error_old, prop_coef, integ_coef, dif_coef, power)
        if left_motor.angle() >= angle_now + angle:
            if (get_reflection_S1() < 17) and (get_reflection_S4() < 17):
                if brake_coef == 1:
                    brake()
                    break
                else:
                    break

def pid_left(angle, brake_coef, power):
    angle_now = left_motor.angle()
    while True:
        pid_drive(integral, error_old, prop_coef, integ_coef, dif_coef, power)
        if left_motor.angle() >= angle_now + angle:
            if (get_reflection_S1() < 15):
                if brake_coef == 1:
                    brake()
                    break
                else:
                    break

def pid_right(angle, brake_coef, power):
    angle_now = left_motor.angle()
    while True:
        pid_drive(integral, error_old, prop_coef, integ_coef, dif_coef, power)
        if left_motor.angle() >= angle_now + angle:
            if (get_reflection_S4() < 15):
                if brake_coef == 1:
                    brake()
                    break
                else:
                    break

def pid_angle(angle, power):
    angle_now = right_motor.angle()
    while not right_motor.angle() >= angle + angle_now:
        pid_drive(integral, error_old, prop_coef*1.5, integ_coef, dif_coef*1.5, power)   
    brake()

def pid_stena(power):
    angle_now = 40
    while not right_motor.angle() == angle_now:
        pid_drive(integral, error_old, prop_coef, integ_coef, dif_coef, power)
        angle_now = right_motor.angle()
        wait(50)
    brake()

def color_filter():
    color = HiTechnic_S2.read('COLOR')
    if (color == (7,)) or (color == (8,)) or (color == (10,)):
        print("RED")
        return "R"
    elif (color == (3,)) or (color == (4,)):
        print("GREEN")
        return "G"
    elif color == (2,):
        print("BLUE")
        return "B"
    else:
        print(color)
        print("XXXXX")
        return "X"

def parkovka():           
    datchiki_open()
    robot.straight(55)
    robot_setup["bunker2"] = color_filter_zabor()
    print(color_filter_zabor())
    robot.straight(95)
    robot_setup["bunker1"] = color_filter_zabor()
    print(color_filter_zabor())
    robot.straight(80)
    datchiki_close()
    robot.straight(35)
    robot_setup["catcher"] = color_filter_zabor()
    print(color_filter_zabor())
    catcher_high()
    robot.straight(-260)

def razvoz_bunker_1(clockwise_turn_1, clockwise_turn_2):
    robot.turn(clockwise_turn_1)
    robot.stop()
    pid_angle(160, 40)
    robot.straight(40)  
    datchiki_open()
    robot.straight(110)
    robot.straight(-122)
    datchiki_close()
    robot.straight(-110)     
    robot.turn(clockwise_turn_2)
    robot.stop()

def razvoz_catcher(clockwise_turn_1, clockwise_turn_2):
    robot.turn(clockwise_turn_1)
    robot.stop()
    pid_angle(160, 40)
    catcher_open()
    robot.straight(35)
    robot.straight(-110)
    robot.turn(clockwise_turn_2)
    robot.stop()
    catcher_polet()

def razvoz_bunker_2(clockwise_turn_1, clockwise_turn_2):
    robot.turn(clockwise_turn_1)
    robot.stop()
    pid_angle(160, 40) 
    datchiki_open()
    robot.straight(210)    
    robot.straight(-230)
    datchiki_close()
    robot.straight(-70)
    robot.turn(clockwise_turn_2)
    robot.stop()

def razvoz(pos_name_left, name_car_left, pos_name_right, name_car_right):
    if (robot_setup["bunker1"] == str(name_car_right))  and  (cars_position[str(pos_name_right)] == 'error_car'):
        razvoz_bunker_1(90, -90)
        robot_setup['bunker1'] = "error_car"
        cars_position[str(pos_name_right)] = str(name_car_right)
    if (robot_setup["bunker1"] == str(name_car_left))  and  (cars_position[str(pos_name_left)] == 'error_car'):
        razvoz_bunker_1(-90, 90)
        robot_setup['bunker1'] = "error_car"
        cars_position[str(pos_name_left)] = str(name_car_left)
    if (robot_setup["bunker1"] == "error_car") and (robot_setup["bunker2"] == str(name_car_right))  and  (cars_position[str(pos_name_right)] == 'error_car'):
        razvoz_bunker_2(90, -90)
        robot_setup['bunker2'] = "error_car"
        cars_position[str(pos_name_right)] = str(name_car_right)
    if (robot_setup["bunker1"] == "error_car") and (robot_setup["bunker2"] == str(name_car_left)) and (cars_position[str(pos_name_left)] == 'error_car'):
        razvoz_bunker_2(-90, 90)
        robot_setup['bunker2'] = "error_car"
        cars_position[str(pos_name_left)] = str(name_car_left)
    if (robot_setup["catcher"] == str(name_car_right))  and  (cars_position[str(pos_name_right)] == 'error_car'):
        razvoz_catcher(90, -90)
        robot_setup['catcher'] = "error_car"
        cars_position[str(pos_name_right)] = str(name_car_right)
    if (robot_setup["catcher"] == str(name_car_left))  and  (cars_position[str(pos_name_left)] == 'error_car'):
        razvoz_catcher(-90, 90)
        robot_setup['catcher'] = "error_car"
        cars_position[str(pos_name_left)] = str(name_car_left)

def zabor_bunker(clockwise_turn_1, clockwise_turn_2):
    robot.turn(clockwise_turn_1)
    robot.stop()
    pid_angle(150, 40)
    wait(100)
    datchiki_open()
    robot.straight(130)
    datchiki_close()
    robot.straight(-200)
    robot.turn(clockwise_turn_2)
    robot.stop()

def zabor_catcher(clockwise_turn_1, clockwise_turn_2):
    catcher_polet()
    robot.turn(clockwise_turn_1)
    robot.stop()
    pid_angle(135, 40)
    robot.straight(50)
    catcher_high()
    robot.straight(-110)
    robot.turn(clockwise_turn_2)
    robot.stop()

def zabor(pos_name_left, name_car_left, pos_name_right, name_car_right):
    if (cars_position[str(pos_name_right)] == "green_car_vid" or cars_position[str(pos_name_right)]  == "blue_car_vid") and robot_setup["bunker2"] == "error_car" and robot_setup["bunker1"] == "error_car":
        zabor_bunker(90, -90)
        if str(name_car_right) == "green_car":
            robot_setup["bunker2"] = "green_car_vid"
        if str(name_car_right) == "blue_car":
            robot_setup["bunker2"] = "blue_car_vid"
        cars_position[str(pos_name_right)] = "error_car"
    if (cars_position[str(pos_name_left)] == "green_car_vid" or cars_position[str(pos_name_left)]  == "blue_car_vid") and robot_setup["bunker2"] == "error_car" and robot_setup["bunker1"] == "error_car":
        zabor_bunker(-90, 90)
        if cars_position[str(pos_name_left)] == "green_car":
            robot_setup["bunker2"] = "green_car_vid"
        else:
            robot_setup["bunker2"] = "blue_car_vid"     
        cars_position[str(pos_name_left)] = "error_car"                           
    if robot_setup["bunker1"] == "error_car" and  (robot_setup["bunker2"] == "green_car_vid" or robot_setup["bunker2"] == "blue_car_vid" or (cars_position[str(pos_name_left)] == "red_car" and name_car_right != "red_car" and robot_setup["bunker2"] != "error_car")):
        if cars_position[str(pos_name_right)] == "green_car_vid": 
            zabor_bunker(90, -90)
            robot_setup["bunker1"] = cars_position[str(pos_name_right)]  
            cars_position[str(pos_name_right)] = "error_car" 
        if cars_position[str(pos_name_right)] == "blue_car_vid":
            zabor_bunker(90, -90)
            robot_setup["bunker1"] = cars_position[str(pos_name_right)]     
            cars_position[str(pos_name_right)] = "error_car" 
        if cars_position[str(pos_name_right)] == "red_car"  and (pos_name_right != "red_pos_1" or pos_name_right != "red_pos_2" or pos_name_right != "red_pos_3" or pos_name_right != "red_pos_4"):
            zabor_bunker(90, -90)   
            robot_setup["bunker1"] = cars_position[str(pos_name_right)]      
            cars_position[str(pos_name_right)] = "error_car"      
    if robot_setup["bunker1"] == "error_car" and (robot_setup["bunker2"] == "green_car_vid" or robot_setup["bunker2"] == "blue_car_vid" or (cars_position[str(pos_name_left)] == "red_car" and name_car_left != "red_car" and robot_setup["bunker2"] != "error_car")):
        if cars_position[str(pos_name_left)] == "green_car_vid": 
            zabor_bunker(-90, 90)
            robot_setup["bunker1"] = cars_position[str(pos_name_left)]      
            cars_position[str(pos_name_left)] = "error_car"
        if cars_position[str(pos_name_left)] == "blue_car_vid":
            zabor_bunker(-90, 90)
            robot_setup["bunker1"] = cars_position[str(pos_name_left)]      
            cars_position[str(pos_name_left)] = "error_car"
        if cars_position[str(pos_name_left)] == "red_car"  and (pos_name_left != "red_pos_1" or pos_name_left != "red_pos_2" or pos_name_left != "red_pos_3" or pos_name_left != "red_pos_4"):
            zabor_bunker(-90, 90)
            robot_setup["bunker1"] = cars_position[str(pos_name_left)]      
            cars_position[str(pos_name_left)] = "error_car"
    if robot_setup["catcher"] == "error_car" and  (cars_position[str(pos_name_right)] == "green_car_vid" or cars_position[str(pos_name_right)] == "blue_car_vid" or (cars_position[str(pos_name_right)] == "red_car" and name_car_right != "red_car")):
        zabor_catcher(90, -90)
        robot_setup["catcher"] = cars_position[str(pos_name_right)]
        cars_position[str(pos_name_right)] = "error_car"
    if robot_setup["catcher"] == "error_car" and  (cars_position[str(pos_name_left)] == "green_car_vid" or cars_position[str(pos_name_left)] == "blue_car_vid" or (cars_position[str(pos_name_left)] == "red_car" and name_car_left != "red_car")):
        zabor_catcher(-90, 90)
        robot_setup["catcher"] = cars_position[str(pos_name_left)]
        cars_position[str(pos_name_left)] = "error_car"

def zamer(pos_name):
    pid_angle(45, 40)
    wait(1000)
    cars_position[str(pos_name)] = color_filter_proezd()
    wait(1000)
    pid_angle(56, 40)

def proezd(pos_name_left, name_car_left, pos_name_right, name_car_right):
    if (name_car_left == "green_car" or name_car_left == "blue_car" or (name_car_left == "red_car" and cars_position[str(pos_name_left)] == "red_car")) and (name_car_right == "green_car" or name_car_right == "blue_car" or (name_car_right == "red_car" and cars_position[str(pos_name_right)] == "red_car")): 
        robot.straight(85)
        robot.stop()
    zabor(pos_name_left, name_car_left, pos_name_right, name_car_right)
    print("CATCHER", robot_setup["catcher"], "BUNKER1", robot_setup["bunker1"], "BUNKER2", robot_setup["bunker2"], "LEFT_POS", cars_position[str(pos_name_left)], "RIGHT_POS", cars_position[str(pos_name_right)])
    razvoz(pos_name_left, name_car_left, pos_name_right, name_car_right)
    print("CATCHER", robot_setup["catcher"], "BUNKER1", robot_setup["bunker1"], "BUNKER2", robot_setup["bunker2"], "LEFT_POS", cars_position[str(pos_name_left)], "RIGHT_POS", cars_position[str(pos_name_right)])
    zabor(pos_name_left, name_car_left, pos_name_right, name_car_right)
    print("CATCHER", robot_setup["catcher"], "BUNKER1", robot_setup["bunker1"], "BUNKER2", robot_setup["bunker2"], "LEFT_POS", cars_position[str(pos_name_left)], "RIGHT_POS", cars_position[str(pos_name_right)])
    razvoz(pos_name_left, name_car_left, pos_name_right, name_car_right)
    print("CATCHER", robot_setup["catcher"], "BUNKER1", robot_setup["bunker1"], "BUNKER2", robot_setup["bunker2"], "LEFT_POS", cars_position[str(pos_name_left)], "RIGHT_POS", cars_position[str(pos_name_right)])
    if (robot_setup["bunker1"] == "green_car_vid" or robot_setup["bunker1"] == "blue_car_vid") and (robot_setup["catcher"] == "green_car_vid" or robot_setup["catcher"] == "blue_car_vid"):
        return True
        
def go_to_vidacha():
    cars_position["vidacha_2"] = robot_setup["bunker2"]
    robot_setup["bunker1"] = "error_car"
    cars_position["vidacha_1"] = robot_setup["bunker1"]
    robot_setup["bunker2"] = "error_car"
    pid_angle(600, 80)
    robot.stop()
    wait(200)
    robot.turn(-90)
    datchiki_open()
    robot.straight(215)
    robot.straight(-120)
    datchiki.dc(50)
    wait(100)
    datchiki.stop(Stop.HOLD)
    robot.straight(-150)
    datchiki_open()
    robot.straight(70)
    datchiki_close()
    robot.stop()
    wait(100)
    robot.turn(90)
    robot.stop()
    pid_T(0, 0, 80)
    pid_angle(500, 80)
    robot.stop()
    wait(100)
    robot.turn(-90)
    robot.stop()
    datchiki_open()
    robot.straight(210)
    robot.straight(-240)
    datchiki_close()
    robot.straight(140)
    robot.straight(-100)
    robot.stop()
    wait(100)
    robot.turn(-90)
    robot.stop()
    pid_left(250, 1, 100)


robot_setup['catcher'] = "error_car"
robot_setup['bunker1'] = "error_car"
robot_setup['bunker2'] = "error_car"

cars_position['blue_pos_1'] = "error_car"
cars_position['blue_pos_2'] = "error_car"
cars_position['blue_pos_3'] = "error_car"
cars_position['blue_pos_4'] = "error_car"

cars_position['red_pos_1'] = "error_car"
cars_position['red_pos_2'] = "error_car"
cars_position['red_pos_3'] = "error_car"
cars_position['red_pos_4'] = "error_car"

cars_position['green_pos_1'] = "error_car"
cars_position['green_pos_2'] = "error_car"
cars_position['green_pos_3'] = "error_car"
cars_position['green_pos_4'] = "error_car"


datchiki.dc(60)
wait(500)
datchiki.stop(Stop.HOLD)
catcher.run_time(600, 1500,then=Stop.HOLD, wait=True)
catcher.reset_angle(0)
catcher_open()
wait(2000)
catcher_high()
position = 3
while True:
    clockwise = 0  # clockwise - расположение робота в конкретную сторону: 0 - в сторорну 8 и 14 зон, 1 - в сторону 9 и 3 зон   
    drive_to(4)
    if proezd("green_pos_2", "green_car", "blue_pos_4", "blue_car"):
        break

    drive_to(5)                                            
    if proezd("red_pos_3", "red_car", "blue_pos_3", "blue_car"):
        break  

    drive_to(6)
    if proezd("green_pos_3", "green_car", "red_pos_4", "red_car"):
        break

    drive_to(7)
    if proezd("blue_pos_2", "blue_car", "green_pos_4", "green_car"):
        break

    if cars_position["blue_pos_2"] != "error_car":
        drive_to(6)
        сlockwise = 1 
        if proezd("red_pos_4", "red_car", "green_pos_3", "green_car"):
            break

        drive_to(5)                                                      
        if proezd("blue_pos_3", "blue_car", "red_pos_3", "red_car"):
            break         

        drive_to(4)
        if proezd("blue_pos_4", "blue_car", "green_pos_2", "green_car"):
            break

        drive_to(10)
        clockwise = 0                                                      
        if proezd("green_pos_2", "green_car", "red_pos_1", "red_car"):         
            break

        drive_to(11)                                                   
        if proezd("red_pos_3", "red_car", "red_pos_2", "red_car"):
            break     

        drive_to(12)                                                   
        if proezd("green_pos_1", "green_car", "green_pos_3", "green_car"):
            break   

        drive_to(13)                                               
        if proezd("blue_pos_1", "blue_car", "blue_pos_2", "blue_car"):    
            break

        robot.turn(180)
        robot.stop()       
        drive_to(12)
        clockwise = 1                                                     
        if proezd("green_pos_1", "green_pos_2", "green_car"):  
            break

        drive_to(11)                                                      
        if proezd("red_pos_2", "red_car", "red_pos_3", "red_car"):  
            break

        drive_to(10)                                                      
        if proezd("red_pos_1", "red_car", "green_pos_2", "green_car"):  
            break

        drive_to(9)
        break
    if cars_position["blue_pos_2"] == "error_car":
        drive_to(13)
        clockwise = 1 
        if proezd("blue_pos_1", "blue_car", "blue_pos_2", "blue_car"):    
            break

        drive_to(12)                                                       
        if proezd("green_pos_1", "green_car", "green_pos_2", "green_car"):  
            break

        drive_to(11)                                                      
        if proezd("red_pos_2", "red_car", "red_pos_3", "red_car"):  
            break

        drive_to(10)                                                      
        if proezd("red_pos_1", "red_car", "green_pos_2", "green_car"):  
            break

        drive_to(9)
        break
    if clockwise == 0:
        robot.turn(180)
        drive_to(3)
    else:
        drive_to(3)

# robot.straight(86) 
# robot.turn(90)
# robot.stop()
# go_to_vidacha()
# robot.straight(90)
# robot.turn(-90)
# robot.stop()
# pid_right(100, 1, 60)
# catcher_polet()
# robot.stop()
# robot.settings(300, 500, 200, 200)
# pid_angle(130, 50)
# wait(1000)
# robot.turn(-85)
# robot.stop()
# wait(1000)
# datchiki_open()
# parkovka()
# robot.turn(90)
# robot.stop()
# pid_T(100, 1, 70)
# robot.straight(90)
# robot.stop()
# robot.settings(1000, 1000, 600, 600)